// Importing necessary functions from D3 library
import { csv, select } from 'd3';
// Importing custom scatterPlot function from local file
import { scatterPlot } from './scatterPlot';

// URL to fetch CSV data
const csvUrl = [
  'https://gist.githubusercontent.com/akhilcharugulla/f1b878fec3f09de215c06cecb79d2226/raw/cedeed891106fb23b4dec107408d2e06345fdb59/schooldata.csv',
].join('');

// Function to parse each row of CSV data
const parseRow = (d) => {
  d.Math_Score = +d.Math_Score;
  d.Science_Score = +d.Science_Score;
  d.English_Score = +d.English_Score;
  d.History_Score = +d.History_Score;
  return d;
};

// Get the width and height of the window
const width = window.innerWidth;
const height = window.innerHeight;

// Append an SVG element to the body
const svg = select('body')
  .append('svg')
  .attr('width', width)
  .attr('height', height);

// Main function to initialize the scatter plot
const main = async () => {
  // Create an instance of scatterPlot
  const plot = scatterPlot()
    .width(width)
    .height(height)
    // Load data from CSV and parse each row
    .data(await csv(csvUrl, parseRow))
    // Define x and y values for scatter plot
    .xValue((d) => d.Math_Score)
    .yValue((d) => d.Science_Score)
    // Set margins for the plot
    .margin({
      top: 50,
      right: 50,
      bottom: 70,
      left: 70,
    })
    // Set radius for each data point
    .radius(5)
    // Set axis labels
    .xLabel('Math Score')
    .yLabel('Science Score');

  // Call the scatter plot function on the SVG element
  svg.call(plot);

  // Array of column names for dynamic data update
  const columns = [
    'Math_Score',
    'Science_Score',
    'English_Score',
    'History_Score',
  ];
  let i = 0;
  // Update data every 2 seconds
  setInterval(() => {
    plot.xValue((d) => d[columns[i % columns.length]]);
    svg.call(plot);
    i++;
  }, 2000);
};

// Call the main function to start the visualization
main();
