// Importing necessary functions from the d3 library
import {
  scaleLinear,
  extent,
  axisLeft,
  axisBottom,
} from 'd3';

// Defining a function named scatterPlot
export const scatterPlot = () => {
  // Declaring variables
  let width;
  let height;
  let data;
  let xValue;
  let yValue;
  let margin;
  let radius;
  let xLabel;
  let yLabel;

  // Defining the main function that will be returned
  const my = (selection) => {
    // Creating x and y scales
    const x = scaleLinear()
      .domain(extent(data, xValue))
      .range([margin.left, width - margin.right]);

    const y = scaleLinear()
      .domain(extent(data, yValue))
      .range([height - margin.bottom, margin.top]);

    // Mapping data to positions and colors
    const marks = data.map((d) => ({
      x: x(xValue(d)),
      y: y(yValue(d)),
    }));

    // Drawing circles for each data point
    const circles = selection
      .selectAll('circle')
      .data(marks)
      .join('circle')
      .attr('cx', (d) => d.x)
      .attr('cy', (d) => d.y)
      .attr('r', radius)
      .attr('stroke', 'white')
      .attr('fill', 'blue');

    // Adding Y-axis and formatting it
    selection
      .selectAll('.y-axis')
      .data([null])
      .join('g')
      .attr('class', 'y-axis')
      .attr('transform', `translate(${margin.left},0)`)
      .attr('fill', 'black')
      .call(axisLeft(y))
      .append('text')
      .attr('x', -margin.left)
      .attr('y', 25) // Adjust the y position for the label
      .attr('fill', 'black')
      .attr('text-anchor', 'start')
      .text(yLabel);

    // Adding X-axis and formatting it
    selection
      .selectAll('.x-axis')
      .data([null])
      .join('g')
      .attr('class', 'x-axis')
      .attr('fill', 'black')
      .attr(
        'transform',
        `translate(0,${height - margin.bottom})`,
      )
      .call(axisBottom(x))
      .append('text')
      .attr('x', width - margin.right)
      .attr('y', -8)
      .attr('fill', 'black')
      .attr('text-anchor', 'end')
      .text(xLabel);
  };

  // Getter/setter functions for variables
  my.width = function (_) {
    return arguments.length ? ((width = +_), my) : width;
  };

  my.height = function (_) {
    return arguments.length ? ((height = +_), my) : height;
  };

  my.data = function (_) {
    return arguments.length ? ((data = _), my) : data;
  };

  my.xValue = function (_) {
    return arguments.length ? ((xValue = _), my) : xValue;
  };

  my.yValue = function (_) {
    return arguments.length ? ((yValue = _), my) : yValue;
  };

  my.margin = function (_) {
    return arguments.length ? ((margin = _), my) : margin;
  };

  my.radius = function (_) {
    return arguments.length ? ((radius = +_), my) : radius;
  };

  my.xLabel = function (_) {
    return arguments.length ? ((xLabel = _), my) : xLabel;
  };

  my.yLabel = function (_) {
    return arguments.length ? ((yLabel = _), my) : yLabel;
  };

  return my; // Returning the main function
};
